
#include <stdio.h>
#include <stdlib.h>
#include "cell.h"
#include "rif.h"

#include "tiffio.h"

#define INTEL

#define HiPic_Header_Size 64

struct HiPic_Header
{
	char					head[2];
	unsigned short			comment_length;
	unsigned short			width;
	unsigned short			height;
	unsigned short			x_offset;
	unsigned short			y_offset;
	unsigned short			type;
	char					reserved[63 - 13];
	char					*comment;
};
typedef struct HiPic_Header Header;

short	*data;

static void Error(msg)
char        *msg;
	{
		fputs(msg,stderr);
		fputc('\n',stderr);
		exit(1);
	}
	
/*----------------------------------------------------------------------*/

void	StoreTiffFile(path,Nx,Ny,BPS,hell,bell,desc)
char	*path,*desc;
short	Nx,Ny,BPS;
unsigned short *hell;
unsigned char *bell;
{
	TIFF	*image;
	long	i, j, k, l;

	image = TIFFOpen(path, "wl");

	TIFFSetField(image, TIFFTAG_IMAGEWIDTH, Nx);
	TIFFSetField(image, TIFFTAG_IMAGELENGTH, Ny);
	TIFFSetField(image, TIFFTAG_BITSPERSAMPLE, BPS);
	TIFFSetField(image, TIFFTAG_COMPRESSION, COMPRESSION_NONE);
	TIFFSetField(image, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK);
	TIFFSetField(image, TIFFTAG_SAMPLESPERPIXEL, 1);
	TIFFSetField(image, TIFFTAG_ROWSPERSTRIP, 1);
//	TIFFSetField(image, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT);// ȂĂ悢
//	TIFFSetField(image, TIFFTAG_FILLORDER, FILLORDER_MSB2LSB);
//	TIFFSetField(image, TIFFTAG_STRIPBYTECOUNTS,h.width*sizeof(short));
//	TIFFSetField(image, TIFFTAG_DATATYPE, TIFF_SHORT);  // ȂĂ悢
	TIFFSetField(image, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
	TIFFSetField(image, TIFFTAG_IMAGEDESCRIPTION, desc);
	TIFFSetField(image, TIFFTAG_ARTIST, "SP-uCT");

	if(BPS==16){
		for(i = 0; i < Ny; ++i)
			TIFFWriteRawStrip(image, i, hell+i*Nx, Nx*sizeof(short));
	}
	else if (BPS==8){
		for(i = 0; i < Ny; ++i){
			TIFFWriteRawStrip(image, i, bell+i*Nx, Nx*sizeof(char));
		}
	}

	TIFFClose(image);
}


/*----------------------------------------------------------------------*/
int read_hipic(fname, h)
char	*fname;
Header	*h;
{
		short		x_size, y_size;
		int		j;
		FILE	*fi;

//open input files
		if((fi = fopen(fname,"rb")) == NULL)
		{
			printf("can not open %s for input\n", fname);
			free(h->comment); 
			return(-10);
		}
// read comment and image data from file1
		if (fread(h, sizeof(char), HiPic_Header_Size, fi) != HiPic_Header_Size)
		{
			fprintf(stderr, "EOF in %s (header)\n", fname);
			free(h->comment); 
			return(-1);
		}
		if (strncmp(h->head, "IM", 2))
		{
			fclose(fi); 
			free(h->comment); 
			fprintf(stderr, "This is not HiPIc image file.\n"); 
			fprintf(stderr, "File name = %s\n", fname); 
			return(-11);
		}

		h->comment = (char *) malloc(h->comment_length + 1);
		if (h->comment == NULL)
		{
			fclose(fi); 
			free(h->comment); 
			fprintf(stderr, "Connot allocate memory (HiPic_Header.comment)\n"); 
			fprintf(stderr, "File name = %s\n", fname); 
			return(-12);
		}
		if (fread(h->comment, sizeof(char), h->comment_length, fi) != h->comment_length)
		{ 
			fclose(fi); 
			free(h->comment); 
			fprintf(stderr, "EOF found (comment)\n"); 
			fprintf(stderr, "File name = %s\n", fname); 
			return(-13);
		}

		h->comment[h->comment_length] = '\0';

		x_size = h->width;
		y_size = h->height;

		data = (short *) malloc(x_size * y_size * sizeof(short));
		if ((j = fread(data, sizeof(short), x_size*y_size, fi)) != x_size*y_size)
		{
			fclose(fi);
			free(h->comment); 
			fprintf(stderr, " Error reading %s at %d\n", fname, j);
			fprintf(stderr, "File name = %s\n", fname); 
			return(1);
		}

// check for correct reading of HiPic file
//printf("-------------------------------\n");
//printf("%s\n", fname);
//printf("%s\n", h->head);
//printf("%d\n", h->comment_length);
//printf("%d\n", h->width);
//printf("%d\n", h->height);
//printf("%d\n", h->x_offset);
//printf("%d\n", h->y_offset);
//printf("%d\n", h->type);
//printf("%s\n", h->reserved);
//printf("%s\n", h->comment);
//printf("-------------------------------\n");

		fclose(fi);
		return(0);
	}

/*----------------------------------------------------------------------*/


int	main(argc,argv)
int	argc;
char	**argv;
{
	int		Nx,Ny,BPS,dBPS;
	short	data_max, data_min;
	long	vv,hh,j, lval, hval, i, l_inv;
	double	dd, dval;
	Cell	**cell;
	char	*desc;
	char	fout[50];
	char	fname[50], tfname[50];
	FILE	*f;

	Header	h;

	unsigned short	*hell;
	unsigned char	*bell;

	if (argc<5) {
	    fputs("usage : simg2tif inputfile low high bit\n",stderr);
	    exit(1);
	}
	lval=atoi(argv[2]);
	hval=atoi(argv[3]);
	dBPS=atoi(argv[4]);


// open parameter file
	if((f = fopen(argv[1],"r")) == NULL){
		printf("cannot open %s \n", argv[1]);
		return(-10);
	}

	i=0;
	while( fscanf(f, "%s", fname) != EOF){
//		printf("%s\n", fname);
		if ((j = read_hipic(fname, &h)) != 0){
			printf("something wrong -- return value is %d", j);
			exit(-1);
		}
		Nx = h.width;
		Ny = h.height;

		data_max = 0;
		data_min = 32000;
		
		for (j=0; j<Nx*Ny;j++){
			if(data_max<*(data+i)) data_max=*(data+i);
			if(data_min>*(data+i)) data_min=*(data+i);
		}

		for(j=0;j<strlen(fname)-3;++j){
			tfname[j]=fname[j];
		}
		tfname[strlen(fname)-3]=0;
		sprintf(fout, "%stif", tfname);
		

//		if (dBPS == 16) printf("%s is converted to %s (16bits).\n", fname, fout);
//		if (dBPS == 8){
//			printf("%s is converted to %s (8bits).\n", fname, fout);
//			printf("from %d to %d\n", lval, hval);
//		}


		// initialize cell and bell
		hell = (unsigned short *)malloc(Nx*Ny*sizeof(short));
		bell = (unsigned char *)malloc(Nx*Ny*sizeof(char));

		j=0;
		dd=255./(double)(hval-lval);
		for (vv=0; vv<Ny; vv++){
			for (hh=0; hh<Nx; hh++){
				if (dBPS ==16){
					*(hell+j)=(unsigned short)(*(data+j));
					*(bell+j)=(unsigned char)((double)255*((double)*(data+j)-(double)lval)/((double)hval-(double)lval));
				}
				if (dBPS == 8){
					dval = ((double)255*((double)*(data+j)-(double)lval)/((double)hval-(double)lval));
					if(dval < 1) dval = 0;
					if(dval > 255) dval = 255;
					*(hell+j)=(unsigned short)(*(data+j));
					*(bell+j)=(unsigned char)(dval);
				}
				j=j+1;
			}
		}
		
		StoreTiffFile(fout,Nx,Ny,dBPS,hell,bell,h.comment);
		
		printf("%s -> %s ", fname, fout);
			if(dBPS==16){
				printf("(16bits) \n");
			}
			if(dBPS==8){
				printf("(8bits) from %d to %d\n", lval, hval);
			}
		
		
		i=i+1;
		free(hell);
		free(bell);
		free(data);
	}
	



	return 0;
}
